# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals


def get_url(*parts):
    return '{}/{}/'.format(base, '/'.join(parts))


protocol = 'http'

domain = 'api.platform.xixiang000.com'

base = '{}://{}'.format(protocol, domain)

login_url = get_url('login')

list_menu_url = get_url('list', 'getMenu')

list_url = get_url('list')

cookbook_url = get_url('cookbook')

cart_add_url = get_url('cart', 'add')

user_address_url = get_url('userAddress')

order_add_url = get_url('order', 'add')



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals


class Model(object):
    @classmethod
    def load(cls, data):
        pass


class Menu(Model):
    def __init__(self, name, menu_type):
        self.name = name
        self.menu_type = menu_type

    @classmethod
    def load(cls, data):
        """
        :rtype: list[Menu]
        """
        menus = []
        for menu_type, name in data.items():
            menus.append(cls(name, menu_type))
        return menus

    def __repr__(self):
        return '{} {}'.format(self.menu_type, self.name)


class Business(Model):
    def __init__(self, data):
        self.business_id = data['business_id']
        self.business_name = data['business_name']
        self.company_id = data['company_id']
        self.raw_data = data

    @classmethod
    def load(cls, instances):
        """
        :rtype: list[Business]
        """
        businesses = []
        for data in instances:
            businesses.append(cls(data))
        return businesses

    def __repr__(self):
        return '{} {}'.format(self.business_id, self.business_name)


class Item(Model):
    def __init__(self, data):
        self.item_id = data['id']
        self.menu_id = data['menu_id']
        self.menu_type = data['menu_type']
        self.is_sell = bool(data['is_sell'])
        self.menu_name = data['cookbook']['menu_name']
        self.raw_data = data

    @classmethod
    def load(cls, data):
        """
        :rtype: list[Item]
        """
        businesses = []
        for items_data in data.values():
            for item_data in items_data:
                businesses.append(cls(item_data))
        return businesses

    def __repr__(self):
        return '{} {}'.format(self.item_id, self.menu_name)


class Address(Model):
    def __init__(self, data):
        self.address_id = data['id']
        self.address = data['consignee_address']
        self.raw_data = data

    @classmethod
    def load(cls, instances):
        """
        :rtype: list[Address]
        """
        addresses = []
        for instance in instances:
            addresses.append(cls(instance))
        return addresses

    def __repr__(self):
        return '{} {}'.format(self.address_id, self.address)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals


class XiXiangError(Exception):
    """ XiXiang 包产生的错误 """


class XiXiangRequestError(XiXiangError):
    """ 网络请求过程中的一些错，比如状态码不是200，code不是1 """


class XiXiangLoginFail(XiXiangError):
    """ 用户名或密码错误，导致的登录失败 """



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from setuptools import setup

import xixiang

with open('README.md', 'rb') as readme_file:
    long_description = readme_file.read().decode('utf-8')

with open('requirements.txt') as requirement_file:
    requirements = [_.strip() for _ in requirement_file.readlines() if _]

setup(
    name='xixiang',
    version=xixiang.__version__,
    description='xixiang order sdk',
    long_description=long_description,
    author='Lirian Su',
    author_email='liriansu@gmail.com',
    url='https://github.com/hui-z/xixiang',
    license='MIT License',
    packages=['xixiang'],
)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import http

import requests
import six

import xixiang


class XiXiang(object):
    @classmethod
    def login(cls, phone, password):
        """
        :rtype: XiXiang
        """
        response = requests.post(xixiang.urls.login_url, {'phone': phone, 'password': password})
        if response.status_code == http.HTTPStatus.OK.value:
            data = response.json()
            if data['code'] == 1:
                return cls(data['data'])
            else:
                raise xixiang.XiXiangLoginFail('{} 登录失败：{}'.format(phone, data['message']))
        elif response.status_code == http.HTTPStatus.UNPROCESSABLE_ENTITY.value:
            raise xixiang.XiXiangLoginFail('{} 登录失败：密码错误'.format(phone))
        raise xixiang.XiXiangLoginFail('{} 登录失败：服务器返回码 {}'.format(phone, response.status_code))

    def __init__(self, api_token):
        self.api_token = api_token

    def get_menus(self):
        return xixiang.Menu.load(self.get(xixiang.urls.list_menu_url))

    def get_businesses(self, menu):
        return xixiang.Business.load(self.get(xixiang.urls.list_url, {'mt': menu.menu_type}))

    def get_items(self, business, menu):
        """
        :type business: xixiang.models.Business
        :type menu: xixiang.models.Menu
        """
        return xixiang.Item.load(self.get(
            xixiang.urls.cookbook_url,
            {
                'busid': business.business_id,
                'comid': business.company_id,
                'mt': menu.menu_type,
            },
        ))

    def add_item(self, item, num=1):
        """
        :type item: xixiang.models.Item
        :type num: int
        """
        return self.post(
            xixiang.urls.cart_add_url,
            {
                'id': item.item_id,
                'menu_id': item.menu_id,
                'mt': item.menu_type,
                'num': num,
                'reserve_date': 'undefined',
            },
        )

    def get_addresses(self):
        return xixiang.Address.load(self.get(xixiang.urls.user_address_url))

    def add_order(self, address):
        return self.post(xixiang.urls.order_add_url, {'address_id': address.address_id})

    def get(self, url, params=None):
        """
        :rtype: dict
        """
        if params is None:
            params = {}
        params['api_token'] = self.api_token
        url = '{}?{}'.format(url, six.moves.urllib.parse.urlencode(sorted(params.items())))
        response = requests.get(url)
        data = response.json()
        if data['code'] == 1:
            return data['data']
        raise xixiang.XiXiangRequestError('请求失败：{}'.format(data['message']))

    def post(self, url, data=None):
        """
        :rtype: dict
        """
        url = '{}?api_token={}'.format(url, self.api_token)
        response = requests.post(url, data or {})
        data = response.json()
        if data['code'] == 1:
            return data['data']
        raise xixiang.XiXiangRequestError('请求失败：{}'.format(data['message']))



# -*- coding: utf-8 -*-

__author__ = 'Lirian Su'
__version__ = '0.1.1'

from . import urls as urls  # NOQA
from .exceptions import *  # NOQA
from .models import *  # NOQA
from .services import XiXiang  # NOQA
